﻿//---------------------------------------------------------------------------
//
//   Lathe
//
//          Copyright(C) 1999-2017, tetraface Inc.
//
//     Sample of Object plug-in for creating a lathe by lines in an object.
//     Created DLL must be installed in "Plugins\Object" directory.
//
//    　オブジェクト中のラインから回転体を作成するオブジェクトプラグインの
//    サンプル。
//    　作成したDLLは"Plugins\Object"フォルダに入れる必要がある。
//
//
//---------------------------------------------------------------------------

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "MQPlugin.h"
#include "MQ3DLib.h"
#include "MQWidget.h"
#include "resource.h"

HINSTANCE hInstance;

BOOL Lathe(MQDocument doc);


//---------------------------------------------------------------------------
//  DllMain
//---------------------------------------------------------------------------
BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
	//ダイアログボックスの表示に必要なので、インスタンスを保存しておく
	hInstance = (HINSTANCE)hModule;

	//プラグインとしては特に必要な処理はないので、何もせずにTRUEを返す
    return TRUE;
}

//---------------------------------------------------------------------------
//  MQGetPlugInID
//    プラグインIDを返す。
//    この関数は起動時に呼び出される。
//---------------------------------------------------------------------------
MQPLUGIN_EXPORT void MQGetPlugInID(DWORD *Product, DWORD *ID)
{
	// プロダクト名(制作者名)とIDを、全部で64bitの値として返す
	// 値は他と重複しないようなランダムなもので良い
	*Product = 0x56A31D20;
	*ID      = 0xE0A5671C;
}

//---------------------------------------------------------------------------
//  MQGetPlugInName
//    プラグイン名を返す。
//    この関数は[プラグインについて]表示時に呼び出される。
//---------------------------------------------------------------------------
MQPLUGIN_EXPORT const char *MQGetPlugInName(void)
{
	// プラグイン名
	return "Lathe           Copyright(C) 1999-2017, tetraface Inc.";
}

//---------------------------------------------------------------------------
//  MQGetPlugInType
//    プラグインのタイプを返す。
//    この関数は起動時に呼び出される。
//---------------------------------------------------------------------------
MQPLUGIN_EXPORT int MQGetPlugInType(void)
{
	// オブジェクト変形用プラグインである
	return MQPLUGIN_TYPE_OBJECT;
}

//---------------------------------------------------------------------------
//  MQEnumString
//    ポップアップメニューに表示される文字列を返す。
//    この関数は起動時に呼び出される。
//---------------------------------------------------------------------------
MQPLUGIN_EXPORT const char *MQEnumString(int index)
{
	switch(index){
	case 0: return "Lathe";
	}
	return NULL;
}

//---------------------------------------------------------------------------
//  MQModifyObject
//    メニューから選択されたときに呼び出される。
//---------------------------------------------------------------------------
MQPLUGIN_EXPORT BOOL MQModifyObject(int index, MQDocument doc)
{
	switch(index){
	case 0: return Lathe(doc);
	}
	return FALSE;
}

class LatheDialog : public MQDialog
{
public:
	LatheDialog(MQWindowBase& parent);
	~LatheDialog();

	MQSpinBox *edit_seg;
};

LatheDialog::LatheDialog(MQWindowBase& parent) : MQDialog(parent)
{
	wchar_t buffer[256];

	LoadStringW(hInstance, IDS_TITLE, buffer, _countof(buffer));
	SetTitle(buffer);

	MQFrame *mainFrame = CreateHorizontalFrame(this);

	MQFrame *paramFrame = CreateHorizontalFrame(mainFrame);
	paramFrame->SetMatrixColumn(2);
	
	LoadStringW(hInstance, IDS_SEGMENT, buffer, _countof(buffer));
	CreateLabel(paramFrame, buffer);
	edit_seg = CreateSpinBox(paramFrame);
	edit_seg->SetMin(1);
	edit_seg->SetMax(10000);
	edit_seg->SetPosition(20);

	MQFrame *sideFrame = CreateVerticalFrame(mainFrame);

	LoadStringW(hInstance, IDS_OK, buffer, _countof(buffer));
	MQButton *okbtn = CreateButton(sideFrame, buffer);
	okbtn->SetDefault(true);
	okbtn->SetModalResult(MQDialog::DIALOG_OK);

	LoadStringW(hInstance, IDS_CANCEL, buffer, _countof(buffer));
	MQButton *cancelbtn = CreateButton(sideFrame, buffer);
	cancelbtn->SetCancel(true);
	cancelbtn->SetModalResult(MQDialog::DIALOG_CANCEL);
}

LatheDialog::~LatheDialog()
{
}

//回転体の作成
BOOL Lathe(MQDocument doc)
{
	int obj_index, face_count, vert_count;
	int face, vert_index[2], new_vert[4];
	int **verts;
	int i,j,s;

	// カレントオブジェクトの取得
	obj_index = doc->GetCurrentObjectIndex();
	if(obj_index < 0)
		return FALSE;
	MQObject obj = doc->GetObject(obj_index);
	if(obj == NULL)
		return FALSE;

	face_count = obj->GetFaceCount();
	vert_count = obj->GetVertexCount();

	//ラインが存在するかどうかのチェック
	for(face=0; face<face_count; face++) {
		if(obj->GetFacePointCount(face) == 2)
			break;
	}
	if(face == face_count)
	{
		MQDialog::MessageWarningBox(MQWindow::GetMainWindow(), 
			L"No lines in current object.", L"Lathe");
		return FALSE;
	}

	// ダイアログボックスを表示
	MQWindow mainwin = MQWindow::GetMainWindow();
	LatheDialog dlg(mainwin);
	if(dlg.Execute() != MQDialog::DIALOG_OK){
		return FALSE;
	}

	int segment = dlg.edit_seg->GetPosition();

	// 追加される頂点のインデックスを記録しておくバッファを確保
	verts = (int **)malloc(sizeof(int *) * vert_count);
	for(i=0; i<vert_count; i++)
		verts[i] = NULL;

	for(face=0; face<face_count; face++)
	{
		//２点の面(ライン)であるかどうか
		if(obj->GetFacePointCount(face) != 2)
			continue;

		obj->GetFacePointArray(face, vert_index);

		// 回転体によって作成される頂点を追加
		for(j=0; j<2; j++)
		{
			if(verts[vert_index[j]] == NULL)
			{
				int *cv = (int *)malloc(sizeof(int) * segment);
				verts[vert_index[j]] = cv;

				MQPoint p = obj->GetVertex(vert_index[j]);

				cv[0] = vert_index[j];
				for(s=1; s<segment; s++)
				{
					double angle = (double)s / (double)segment * 2 * PI;
					cv[s] = obj->AddVertex(MQPoint(
						+ p.x * (float)cos(angle) + p.z * (float)sin(angle),
						p.y,
						- p.x * (float)sin(angle) + p.z * (float)cos(angle)));
				}
	
			}
		}

		// 面を両面として作成
		for(s=0; s<segment; s++)
		{
			new_vert[0] = verts[vert_index[0]][s];
			new_vert[1] = verts[vert_index[0]][(s+1)%segment];
			new_vert[2] = verts[vert_index[1]][(s+1)%segment];
			new_vert[3] = verts[vert_index[1]][s];
			obj->AddFace(4, new_vert);

			new_vert[3] = verts[vert_index[0]][s];
			new_vert[2] = verts[vert_index[0]][(s+1)%segment];
			new_vert[1] = verts[vert_index[1]][(s+1)%segment];
			new_vert[0] = verts[vert_index[1]][s];
			obj->AddFace(4, new_vert);
		}

		// オリジナルのラインを削除
		obj->DeleteFace(face);
	}

	// 不要になったバッファを削除
	for(i=0; i<vert_count; i++) {
		if(verts[i] != NULL)
			free(verts[i]);
	}
	free(verts);
	
	return TRUE;
}

