
#include "StringUtil.h"
#include <codecvt>
#include <locale>

namespace StringUtil {

	std::string WideToShiftJis(const std::wstring& wstr)
	{
		NSString *nsstr = [[NSString alloc] initWithBytes:wstr.c_str() length:wstr.length()*sizeof(wchar_t) encoding:NSUTF32LittleEndianStringEncoding];
		NSData* sjisData = [nsstr dataUsingEncoding:NSShiftJISStringEncoding allowLossyConversion:YES];
		const char *chars = static_cast<const char*>([sjisData bytes]);
		return std::string(chars, (size_t)[sjisData length]);
	}

	std::wstring ShiftJisToWide(const std::string& ansistr)
	{
		NSData *sjisData = [NSData dataWithBytes:ansistr.c_str() length:ansistr.length()+1];
		NSString *nsstr = [[NSString alloc] initWithData:sjisData encoding:NSShiftJISStringEncoding];
		const char *chars = [nsstr cStringUsingEncoding:NSUTF8StringEncoding];
		return Utf8ToWide(chars);
	}

	std::wstring Utf8ToWide(const std::string& utf8str)
	{
		try {
			std::wstring_convert<std::codecvt_utf8<wchar_t>, wchar_t> conv;
			return conv.from_bytes(utf8str);
		}catch(...){
		}
		return std::wstring();
	}

	std::string WideToUtf8(const std::wstring& wstr)
	{
		try {
			std::wstring_convert<std::codecvt_utf8_utf16<wchar_t>, wchar_t> conv;
			return conv.to_bytes(wstr);
		}catch(...){
		}
		return std::string();
	}
	
}

